package com.qmth.distributed.print.api;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.dto.BlurryUserDto;
import com.qmth.distributed.print.business.bean.dto.UserDto;
import com.qmth.distributed.print.business.bean.params.UserSaveParams;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.service.SysUserService;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.security.NoSuchAlgorithmException;
import java.util.List;

/**
 * <p>
 * 用户表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "用户Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/user")
@Aac(auth = BOOL.FALSE)
public class SysUserController {
    private final static Logger log = LoggerFactory.getLogger(SysUserController.class);

    @Autowired
    private SysUserService sysUserService;

//    @Resource
//    BasicAttachmentService basicAttachmentService;
//
//    @Resource
//    TBTaskService tbTaskService;

    /**
     * 查询
     *
     * @param loginName
     * @param roleId
     * @param enable
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(@RequestParam(value = "loginName", required = false) String loginName,
                       @RequestParam(value = "roleId", required = false) String roleId,
                       @RequestParam(value = "enable", required = false) Boolean enable,
                       @RequestParam(value = "pageNumber", required = true) Integer pageNumber,
                       @RequestParam(value = "pageSize", required = true) Integer pageSize) {
        IPage<UserDto> userDtos = sysUserService.list(loginName, roleId, enable, pageNumber, pageSize);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 查询
     *
     * @param realName
     * @param enable
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list_customer", method = RequestMethod.POST)
    public Result listCustomer(@RequestParam(value = "realName", required = false) String realName,
                               @RequestParam(value = "enable", required = false) Boolean enable,
                               @RequestParam(value = "pageNumber", required = true) Integer pageNumber,
                               @RequestParam(value = "pageSize", required = true) Integer pageSize) {
        IPage<UserDto> userDtos = sysUserService.listCustomer(realName, enable, pageNumber, pageSize);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 根据科目查询命题老师
     *
     * @param courseCode
     * @param param
     * @return
     */
    @ApiOperation(value = "根据科目查询命题老师")
    @RequestMapping(value = "/user_list", method = RequestMethod.POST)
    public Result userQuery(@RequestParam(value = "courseCode", required = false) String courseCode,
                            @RequestParam(value = "param", required = false) String param) {
        List<BlurryUserDto> list = sysUserService.listUser(courseCode, param);
        return ResultUtil.ok(list);
    }

    /**
     * 新增/修改
     *
     * @param userSaveParams
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(@ApiParam(value = "用户信息", required = true) @Valid @RequestBody UserSaveParams userSaveParams, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        return ResultUtil.ok(sysUserService.saveUser(userSaveParams));
    }

    /**
     * 新增客服
     *
     * @param userSaveParams
     * @return
     */
    @ApiOperation(value = "新增客服")
    @RequestMapping(value = "/save_customer", method = RequestMethod.POST)
    public Result saveCustomer(@ApiParam(value = "用户信息", required = true) @Valid @RequestBody UserSaveParams userSaveParams, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        return ResultUtil.ok(sysUserService.saveCustomer(userSaveParams));
    }

    /**
     * 启用/禁用
     *
     * @param user
     * @return
     */
    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable", method = RequestMethod.POST)
    public Result enable(@RequestBody SysUser user) throws NoSuchAlgorithmException {
        return ResultUtil.ok(sysUserService.enable(user));
    }

    /**
     * 重置密码
     *
     * @param user
     * @return
     */
    @ApiOperation(value = "重置密码")
    @RequestMapping(value = "/reset_password", method = RequestMethod.POST)
    public Result resetPassword(@RequestBody SysUser user) throws NoSuchAlgorithmException {
        return ResultUtil.ok(sysUserService.resetPassword(user.getId()));
    }

    /**
     * 修改密码
     *
     * @return
     */
    @ApiOperation(value = "修改密码")
    @RequestMapping(value = "/update_password", method = RequestMethod.POST)
    public Result updatePassword(@RequestBody SysUser user) throws NoSuchAlgorithmException {
        return ResultUtil.ok(sysUserService.updatePassword(user));
    }

    /**
     * 绑定角色
     *
     * @param sysUser
     * @return
     */
    @ApiOperation(value = "绑定角色")
    @RequestMapping(value = "/bind_roles", method = RequestMethod.POST)
    public Result bindRoles(@RequestBody SysUser sysUser) {
        boolean isSuccess = sysUserService.bindRoles(sysUser);
        return ResultUtil.ok(isSuccess);
    }

//    @Resource
//    AsyncCreatePdfTempleteService asyncCreatePdfTempleteService;
//
//    @Resource
//    OssUtil ossUtil;
//
//    @ApiOperation(value = "异步创建pdf接口")
//    @RequestMapping(value = "/async/create_pdf", method = RequestMethod.POST)
//    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
//    public Result createPdfAysnc(@ApiParam(value = "印刷计划id", required = true) @RequestParam String printPlanId,
//                                 @ApiParam(value = "考务计划id", required = false) @RequestParam(required = false) String examDetailId) throws IOException {
//        Map<String, Object> map = tbTaskService.saveTask(TaskTypeEnum.CREATE_PDF, SystemConstant.convertIdToLong(printPlanId));
//        if (Objects.nonNull(examDetailId)) {
//            map.computeIfAbsent("examDetailId", v -> SystemConstant.convertIdToLong(examDetailId));
//        }
//        map.computeIfAbsent("schoolId", v -> schoolId);
//        asyncCreatePdfTempleteService.createPdf(map, null);
//        TBTask tbTask = Objects.nonNull(map.get(SystemConstant.TASK)) ? (TBTask) map.get(SystemConstant.TASK) : null;
//        return Objects.nonNull(tbTask) ? ResultUtil.ok(new EditResult(tbTask.getId())) : ResultUtil.error("创建任务失败");
//    }
}

